package tetris.model;

import tetris.gui.Block;

/**
 * The class Figure implements a figure of the Tetris game.
 *
 * @author Stephan Fischli
 * @version 2.1
 */
public class Figure {

	/**
	 * The color of the figure.
	 */
	private static final int COLOR = 1;

	/**
	 * The blocks of the figure.
	 */
	private final Block[] blocks = new Block[4];

	/**
	 * Creates a figure at the specified position.
	 *
	 * @param x the x coordinate of the position
	 * @param y the y coordinate of the position
	 */
	public Figure(int x, int y) {
		blocks[0] = new Block(x, y, COLOR);
		blocks[1] = new Block(x - 1, y, COLOR);
		blocks[2] = new Block(x + 1, y, COLOR);
		blocks[3] = new Block(x + 2, y, COLOR);
	}

	/**
	 * Gets the blocks of the figure.
	 *
	 * @return the blocks of the figure
	 */
	public Block[] getBlocks() {
		return blocks;
	}

	/**
	 * Shifts the figure in the specified direction.
	 *
	 * @param dx the x coordinate of the direction
	 * @param dy the y coordinate of the direction
	 */
	public void shift(int dx, int dy) {
		for (Block block : blocks) {
			block.x += dx;
			block.y += dy;
		}
	}

	/**
	 * Rotates the figure in the specified direction.
	 *
	 * @param d the direction of rotation (+1 right, -1 left)
	 */
	public void rotate(int d) {
		int cx = blocks[0].x;
		int cy = blocks[0].y;
		for (Block block : blocks) {
			int dx = block.x - cx;
			int dy = block.y - cy;
			block.x = cx + d * dy;
			block.y = cy - d * dx;
		}
	}
}
