package tetris.model;

import tetris.Tetris;
import tetris.gui.ActionEvent;
import tetris.gui.GUI;

/**
 * The class Game implements the Tetris game.
 *
 * @author Stephan Fischli
 * @version 2.1
 */
public class Game {

	/**
	 * The graphical user interface.
	 */
	private final GUI gui;

	/**
	 * The figure of the game.
	 */
	private Figure figure;

	/**
	 * Constructs a game with a graphical user interface.
	 *
	 * @param gui the graphical user interface
	 */
	public Game(GUI gui) {
		this.gui = gui;
	}

	/**
	 * Starts the game by creating a figure and waiting for action events.
	 */
	public void start() {
		createFigure();
		while (true) {
			ActionEvent event = gui.waitEvent();
			handleEvent(event);
		}
	}

	/**
	 * Creates a figure at the top of the field.
	 */
	private void createFigure() {
		figure = new Figure((Tetris.WIDTH - 1) / 2, Tetris.HEIGHT - 1);
		updateGUI();
	}

	/**
	 * Handles an action event by moving the figure accordingly.
	 *
	 * @param event the event to handle
	 */
	private void handleEvent(ActionEvent event) {
		switch (event) {
			case SHIFT_DOWN -> figure.shift(0, -1);
			case SHIFT_LEFT -> figure.shift(-1, 0);
			case SHIFT_RIGHT -> figure.shift(1, 0);
			case ROTATE_LEFT -> figure.rotate(-1);
			case ROTATE_RIGHT -> figure.rotate(1);
		}
		updateGUI();
	}

	/**
	 * Updates the graphical user interface according to the current state of the game.
	 */
	private void updateGUI() {
		gui.clear();
		gui.drawBlocks(figure.getBlocks());
	}
}
